/***************************************************************************
 *   Copyright (C) 2003-2004 by Flameeyes                                  *
 *   dgp85@users.sourceforge.net                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "kvba.h"
#include "addfavorites.h"
#include "editfavorites.h"
#include "joyconf.h"

#include "settings.h"
#include "uisettingsexec.h"
#include "uisettingsfeatures.h"
#include "uisettingsgraphics.h"

#include <qdir.h>
#include <qstringlist.h>
#include <qtabwidget.h>
#include <qprocess.h>
#include <qlineedit.h>
#include <qlayout.h>

#include <klocale.h>
#include <kconfig.h>
#include <kfiletreeview.h>
#include <kglobal.h>
#include <kiconloader.h>
#include <klistview.h>
#include <klistbox.h>
#include <kaction.h>
#include <ktextbrowser.h>
#include <kurlrequester.h>
#include <kconfigdialog.h>

KvbaMain::KvbaMain(QWidget *parent, const char* name)
    : uiMain( parent, name )
{

	allDirs = new KFileTreeView( tab, "allDirs" );
	allDirs->addColumn( tr2i18n( "Directory" ) );
	allDirs->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)7, (QSizePolicy::SizeType)7, 1, 0, allDirs->sizePolicy().hasHeightForWidth() ) );
	allDirs->setResizePolicy( QListView::AutoOneFit );
	allDirs->setRootIsDecorated( true );
	
	KFileTreeBranch *home = allDirs->addBranch(QDir::home().path(), "Home", KGlobal::iconLoader()->loadIcon("folder_home", KIcon::Small));
	allDirs->setDirOnlyMode(home, true);
	root = allDirs->addBranch("file:/", "Root", KGlobal::iconLoader()->loadIcon("folder_red", KIcon::Small));
	allDirs->setDirOnlyMode(root, true);
	
	QListViewItem *tvi = allDirs->findItem(root, Settings::startDirectory());
	allDirs->setSelected(tvi, true);
	
	tabLayout->insertWidget( 0, allDirs );
	connect( allDirs, SIGNAL( selectionChanged(QListViewItem*) ), this, SLOT( dirSelected(QListViewItem*) ) );

	vba = new QProcess (this);
	connect(vba, SIGNAL(readyReadStdout()), this, SLOT(vbaReadyStdout()));
	connect(vba, SIGNAL(readyReadStderr()), this, SLOT(vbaReadyStderr()));
		
	KGlobal::config()->setGroup("favorites");
		QMap<QString, QString> entries = KGlobal::config()->entryMap("favorites");
		for(QMap<QString, QString>::iterator it = entries.begin(); it != entries.end(); it++)
		{
			QStringList fav = QStringList::split(',', *it, true);
	
			QListViewItem *el = new QListViewItem(favorites);
			int i = 0;
			for(QStringList::iterator it2 = fav.begin(); it2 != fav.end(); it2++, i++)
				el->setText(i, *it2);
			favorites->insertItem( el );
		}
}

KvbaMain::~KvbaMain()
{
	delete vba;
	delete allDirs;
}

void KvbaMain::dirSelected(QListViewItem *i)
{
	allRoms->clear();
	
	allRoms->insertStringList(
		QDir(
			allDirs->currentURL().path(),
			"*.zip;*.gba;*.gbc;*.gb",
			QDir::Name
			).entryList()
		);
}

void KvbaMain::romDoubleClicked(QListBoxItem *i)
{
	vbaOutput->clear();
	vba->clearArguments();
	
	QListViewItem *tvi = allDirs->findItem(root, Settings::startDirectory());
	allDirs->setSelected(tvi, true);
	vba->addArgument(Settings::vBAExec());
	vbaPrepareArguments();
	vba->addArgument(allDirs->currentURL().path() + i->text());
	vba->start();
	
	romsView->setCurrentPage(2);
}

void KvbaMain::vbaReadyStdout()
{
	vbaOutput->append(vba->readLineStdout());
}

void KvbaMain::vbaReadyStderr()
{
	vbaOutput->append("<font color='#FF0000'>" + vba->readLineStderr() + "</font>");
}

void KvbaMain::prefActivated()
{
	if(KConfigDialog::showDialog("settings"))
		return;
	
	KConfigDialog *dialog = new KConfigDialog(this, "settings", Settings::self());
	dialog->addPage(new uiSettingsExec(0, "SettingsExec"), i18n("Executable & Paths"), "package_settings");
	dialog->addPage(new uiSettingsFeatures(0, "SettingsFeatures"), i18n("Features"), "package_games");
	dialog->addPage(new uiSettingsGraphics(0, "SettingsGraphics"), i18n("Graphics"), "package_graphics");
	dialog->show();
}

void KvbaMain::vbaPrepareArguments()
{
	if ( !Settings::biosFile().isEmpty() )
    	vba->addArgument("--bios=" + Settings::biosFile());

	if ( !Settings::configureFile().isEmpty() )
		vba->addArgument("--config=" + Settings::configureFile());
	
	if ( Settings::enableRTC() )
		vba->addArgument("--rtc");
	else
		vba->addArgument("--no-rtc");
	
	if ( Settings::enableAGB() )
		vba->addArgument("--agb-print");
	else
		vba->addArgument("--no-agb-print");
	
	if ( Settings::pauseInactive() ) {
		if ( Settings::version() >= 2 )
			vba->addArgument("--pause-when-inactive");
		else
			vba->addArgument("--pause-when-inative");
	} else
		vba->addArgument("--no-pause-when-inactive");
	
	switch ( Settings::emuSpeed() )
	{
		case 0: vba->addArgument("--no-show-speed"); break;
		case 1: vba->addArgument("--show-speed-normal"); break;
		case 2: vba->addArgument("--show-speed-detailed"); break;
	}
	
	switch ( Settings::flashSize() )
	{
		case 0: vba->addArgument("--flash-64k"); break;
		case 1: vba->addArgument("--flash-128k"); break;
	}
	
	if ( Settings::enableThrottle() )
		vba->addArgument("--throttle=" + QString::number(Settings::throttle()));
	else
		vba->addArgument("--no-throttle");
	
	switch ( Settings::saveType() )
	{
		case 0: vba->addArgument("--save-auto"); break;
		case 1: vba->addArgument("--save-eeprom"); break;
		case 2: vba->addArgument("--save-sram"); break;
		case 3: vba->addArgument("--save-flash"); break;
		case 4: vba->addArgument("--save-sensor"); break;
	}
	
	switch ( Settings::zoom() )
	{
		case 0: vba->addArgument("--video-1x"); break;
		case 1: vba->addArgument("--video-2x"); break;
		case 2: vba->addArgument("--video-3x"); break;
		case 3: vba->addArgument("--video-4x"); break;
	}
	
	if ( Settings::autoFrameSkip() )
		vba->addArgument("--auto-frameskip");
	else
		vba->addArgument("--no-auto-frameskip");

	if ( Settings::enableFullScreen() )
		vba->addArgument("--fullscreen");
	
	vba->addArgument("--frameskip=" + QString::number(Settings::frameSkip()));
	
	switch ( Settings::filter() )
	{
		case  0: vba->addArgument("--filter-normal"); break;
		case  1: vba->addArgument("--filter-tv-mode"); break;
		case  2: vba->addArgument("--filter-2xsai"); break;
		case  3: vba->addArgument("--filter-super-2xsai"); break;
		case  4: vba->addArgument("--filter-super-eagle"); break;
		case  5: vba->addArgument("--filter-pixelate"); break;
		case  6: vba->addArgument("--filter-motion-blur"); break;
		case  7: vba->addArgument("--filter-advmame"); break;
		case  8: vba->addArgument("--filter-simple2x"); break;
		case  9: vba->addArgument("--filter-bilinear"); break;
		case 10: vba->addArgument("--filter-bilinear+"); break;
		case 11: vba->addArgument("--filter-scanlines"); break;
	}
	
	if ( Settings::enableYUV() )
		vba->addArgument("--yuv=" + QString::number(Settings::overlay()));
	
	switch ( Settings::interframe() )
	{
		case 0: vba->addArgument("--ifb-none"); break;
		case 1: vba->addArgument("--ifb-motion-blur"); break;
		case 2: vba->addArgument("--ifb-smart"); break;
	}
}

void KvbaMain::addFavorites()
{
	AddFavorites *addfav = new AddFavorites(this, "addfavorites");
	addfav->romPath->setURL(allDirs->currentURL().path());
	addfav->exec();
	
	delete addfav;
}

void KvbaMain::favDoubleClicked(QListViewItem *i)
{
	vba->clearArguments();
	vba->addArgument(Settings::vBAExec());

	if ( ! i->text(2).isEmpty() )
		vba->addArgument("--ips=" + i->text(2));

	if ( ! i->text(3).isEmpty() )
		vba->addArgument("--config=" + i->text(3));
	else
		vbaPrepareArguments();
		

	vba->addArgument(i->text(1));
	
	vbaOutput->clear();
	vba->start();
	
	romsView->setCurrentPage(2);
}

void KvbaMain::fileNewConfigActivated()
{
	JoyConf *joyconf = new JoyConf(this, "joyconf");
	joyconf->exec();
	delete joyconf;
}

void KvbaMain::favChangeSelection()
{
	editFavoritesAction->setEnabled(true);
}

void KvbaMain::editFavorites()
{
	EditFavorites *editfav = new EditFavorites(this, "editfavorites");
	editfav->gameName->setText(favorites->currentItem()->text(0));
	editfav->romPath->setURL(favorites->currentItem()->text(1));
	editfav->ipsPath->setURL(favorites->currentItem()->text(2));
	editfav->cfgPath->setURL(favorites->currentItem()->text(3));
	editfav->favnum = favorites->itemPos(favorites->currentItem());
	editfav->exec();
	
	delete editfav;
}

#include "kvba.moc"
